@@ -1,4 +1,4 @@
-//! Build and publish crates with pyo3, rust-cpython and cffi bindings as well as rust binaries
+//! Build and publish crates with pyo3, cffi and uniffi bindings as well as rust binaries
 //! as python packages. This file contains the CLI and keyring integration.
 //!
 //! Run with --help for usage information
@@ -16,11 +16,16 @@ use maturin::{
     develop, write_dist_info, BridgeModel, BuildOptions, CargoOptions, DevelopOptions, PathWriter,
     PlatformTag, PythonInterpreter, Target,
 };
+#[cfg(feature = "schemars")]
+use maturin::{generate_json_schema, GenerateJsonSchemaOptions};
 #[cfg(feature = "upload")]
 use maturin::{upload_ui, PublishOpt};
 use std::env;
 use std::path::PathBuf;
-use tracing::debug;
+use std::str::FromStr;
+use tracing::{debug, instrument};
+use tracing_subscriber::filter::Directive;
+use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt, EnvFilter, Layer};
 
 #[derive(Debug, Parser)]
 #[command(
@@ -30,10 +35,29 @@ use tracing::debug;
     after_help = "Visit https://maturin.rs to learn more about maturin.",
     styles = cargo_options::styles(),
 )]
-#[cfg_attr(clippy, allow(clippy::large_enum_variant))]
-/// Build and publish crates with pyo3, rust-cpython and cffi bindings as well
+/// Build and publish crates with pyo3, cffi and uniffi bindings as well
 /// as rust binaries as python packages
-enum Opt {
+struct Opt {
+    /// Use verbose output.
+    ///
+    /// * Default: Show build information and `cargo build` output.
+    /// * `-v`: Use `cargo build -v`.
+    /// * `-vv`: Show debug logging and use `cargo build -vv`.
+    /// * `-vvv`: Show trace logging.
+    ///
+    /// You can configure fine-grained logging using the `RUST_LOG` environment variable.
+    /// (<https://docs.rs/tracing-subscriber/latest/tracing_subscriber/filter/struct.EnvFilter.html#directives>)
+    #[arg(global = true, action = clap::ArgAction::Count, long, short)]
+    verbose: u8,
+    #[command(subcommand)]
+    command: Command,
+}
+
+#[derive(Debug, Parser)]
+#[allow(clippy::large_enum_variant)]
+/// Build and publish crates with pyo3, cffi and uniffi bindings as well
+/// as rust binaries as python packages
+enum Command {
     #[command(name = "build", alias = "b")]
     /// Build the crate into python packages
     Build {
@@ -141,6 +165,10 @@ enum Opt {
     #[cfg(feature = "zig")]
     #[command(subcommand, hide = true)]
     Zig(Zig),
+    /// Generate the JSON schema for the `pyproject.toml` file.
+    #[cfg(feature = "schemars")]
+    #[command(name = "generate-json-schema", hide = true)]
+    GenerateJsonSchema(GenerateJsonSchemaOptions),
 }
 
 /// Backend for the PEP 517 integration. Not for human consumption
@@ -245,7 +273,12 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             strip,
         } => {
             assert_eq!(build_options.interpreter.len(), 1);
-            let context = build_options.into_build_context(true, strip, false)?;
+            let context = build_options
+                .into_build_context()
+                .release(true)
+                .strip(strip)
+                .editable(false)
+                .build()?;
 
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
             let tags = match context.bridge() {
@@ -270,7 +303,12 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             strip,
             editable,
         } => {
-            let build_context = build_options.into_build_context(true, strip, editable)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(true)
+                .strip(strip)
+                .editable(editable)
+                .build()?;
             let wheels = build_context.build_wheels()?;
             assert_eq!(wheels.len(), 1);
             println!("{}", wheels[0].0.to_str().unwrap());
@@ -283,11 +321,20 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
                 out: Some(sdist_directory),
                 cargo: CargoOptions {
                     manifest_path,
+                    // Enable all features to ensure all optional path dependencies are packaged
+                    // into source distribution
+                    all_features: true,
                     ..Default::default()
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(false)
+                .strip(false)
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             let (path, _) = build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
@@ -298,10 +345,8 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
     Ok(())
 }
 
+#[instrument]
 fn run() -> Result<()> {
-    #[cfg(feature = "log")]
-    tracing_subscriber::fmt::init();
-
     #[cfg(feature = "zig")]
     {
         // Allow symlink `maturin` to `ar` to invoke `zig ar`
@@ -323,14 +368,21 @@ fn run() -> Result<()> {
     #[cfg(feature = "wild")]
     let opt = Opt::parse_from(wild::args_os());
 
-    match opt {
-        Opt::Build {
+    setup_logging(opt.verbose)?;
+
+    match opt.command {
+        Command::Build {
             build,
             release,
             strip,
             sdist,
         } => {
-            let build_context = build.into_build_context(release, strip, false)?;
+            let build_context = build
+                .into_build_context()
+                .release(release)
+                .strip(strip)
+                .editable(false)
+                .build()?;
             if sdist {
                 build_context
                     .build_source_distribution()?
@@ -340,14 +392,19 @@ fn run() -> Result<()> {
             assert!(!wheels.is_empty());
         }
         #[cfg(feature = "upload")]
-        Opt::Publish {
+        Command::Publish {
             build,
             mut publish,
             debug,
             no_strip,
             no_sdist,
         } => {
-            let build_context = build.into_build_context(!debug, !no_strip, false)?;
+            let build_context = build
+                .into_build_context()
+                .release(!debug)
+                .strip(!no_strip)
+                .editable(false)
+                .build()?;
 
             if !build_context.release {
                 eprintln!("⚠️  Warning: You're publishing debug wheels");
@@ -365,10 +422,10 @@ fn run() -> Result<()> {
 
             upload_ui(&items, &publish)?
         }
-        Opt::ListPython { target } => {
+        Command::ListPython { target } => {
             let found = if target.is_some() {
                 let target = Target::from_target_triple(target)?;
-                PythonInterpreter::find_by_target(&target, None)
+                PythonInterpreter::find_by_target(&target, None, None)
             } else {
                 let target = Target::from_target_triple(None)?;
                 // We don't know the targeted bindings yet, so we use the most lenient
@@ -379,34 +436,43 @@ fn run() -> Result<()> {
                 eprintln!(" - {interpreter}");
             }
         }
-        Opt::Develop(develop_options) => {
+        Command::Develop(develop_options) => {
             let target = Target::from_target_triple(develop_options.cargo_options.target.clone())?;
             let venv_dir = detect_venv(&target)?;
             develop(develop_options, &venv_dir)?;
         }
-        Opt::SDist { manifest_path, out } => {
+        Command::SDist { manifest_path, out } => {
             let build_options = BuildOptions {
                 out,
                 cargo: CargoOptions {
                     manifest_path,
+                    // Enable all features to ensure all optional path dependencies are packaged
+                    // into source distribution
+                    all_features: true,
                     ..Default::default()
                 },
                 ..Default::default()
             };
-            let build_context = build_options.into_build_context(false, false, false)?;
+            let build_context = build_options
+                .into_build_context()
+                .release(false)
+                .strip(false)
+                .editable(false)
+                .sdist_only(true)
+                .build()?;
             build_context
                 .build_source_distribution()?
                 .context("Failed to build source distribution, pyproject.toml not found")?;
         }
-        Opt::Pep517(subcommand) => pep517(subcommand)?,
+        Command::Pep517(subcommand) => pep517(subcommand)?,
         #[cfg(feature = "scaffolding")]
-        Opt::InitProject { path, options } => init_project(path, options)?,
+        Command::InitProject { path, options } => init_project(path, options)?,
         #[cfg(feature = "scaffolding")]
-        Opt::NewProject { path, options } => new_project(path, options)?,
+        Command::NewProject { path, options } => new_project(path, options)?,
         #[cfg(feature = "scaffolding")]
-        Opt::GenerateCI(generate_ci) => generate_ci.execute()?,
+        Command::GenerateCI(generate_ci) => generate_ci.execute()?,
         #[cfg(feature = "upload")]
-        Opt::Upload { mut publish, files } => {
+        Command::Upload { mut publish, files } => {
             if files.is_empty() {
                 eprintln!("⚠️  Warning: No files given, exiting.");
                 return Ok(());
@@ -416,15 +482,17 @@ fn run() -> Result<()> {
             upload_ui(&files, &publish)?
         }
         #[cfg(feature = "cli-completion")]
-        Opt::Completions { shell } => {
+        Command::Completions { shell } => {
             shell.generate(&mut Opt::command(), &mut std::io::stdout());
         }
         #[cfg(feature = "zig")]
-        Opt::Zig(subcommand) => {
+        Command::Zig(subcommand) => {
             subcommand
                 .execute()
                 .context("Failed to run zig linker wrapper")?;
         }
+        #[cfg(feature = "schemars")]
+        Command::GenerateJsonSchema(args) => generate_json_schema(args)?,
     }
 
     Ok(())
@@ -452,6 +520,33 @@ fn setup_panic_hook() {
     }));
 }
 
+fn setup_logging(verbose: u8) -> Result<()> {
+    // `RUST_LOG` takes precedence over these
+    let default_directive = match verbose {
+        // `-v` runs `cargo build -v`, but doesn't show maturin debug logging yet.
+        0..=1 => tracing::level_filters::LevelFilter::OFF.into(),
+        2 => Directive::from_str("debug").unwrap(),
+        3.. => Directive::from_str("trace").unwrap(),
+    };
+
+    let filter = EnvFilter::builder()
+        .with_default_directive(default_directive)
+        .from_env()
+        .context("Invalid RUST_LOG directives")?;
+
+    let logger = tracing_subscriber::fmt::layer()
+        // Avoid showing all the details from the spans
+        .compact()
+        // Log the timing of each span
+        .with_span_events(tracing_subscriber::fmt::format::FmtSpan::CLOSE);
+
+    tracing_subscriber::registry()
+        .with(logger.with_filter(filter))
+        .init();
+
+    Ok(())
+}
+
 fn main() {
     #[cfg(not(debug_assertions))]
     setup_panic_hook();
